public class Soluzione {

	public static void main (String[] args)	{
		int numClienti = 0; // numero clienti (parametro)
		int numDistributori = 0; // numero distributori (parametro)

		// NOTA: se viene chiesto di parametrizzare un sistema, si devono
		// passare gli argomenti al main, come fatto in questa soluzione.

		if (args.length < 2) {
		  System.out.println("Usage: java Soluzione numDistributori numClienti");
		  return;
		}

		numDistributori = Integer.parseInt(args[0]);
		numClienti = Integer.parseInt(args[1]);

		System.out.println("Starting App");

		// creazione distributori
		Distributore[] dist = new Distributore[numDistributori];
		for ( int i=0 ; i < dist.length ; i++ ) {
			dist[i] = new Distributore("dist" + (i+1));
		}
		// creazione dei clienti
		for ( int i=1 ; i <= numClienti ; i++ ) {
			new Cliente ("cliente" + i, dist).start();
		}
		// coerentemente al modello creo una cisterna
		new Cisterna(dist).start();
	}
}

class Cliente extends Thread {

	static final int maxRich = 3; // max richiesta per cliente
	Distributore[] dist;

	public Cliente (String nome, Distributore[] dist) 	{
		super(nome);
		this.dist = dist;
	}

	public void run () {
		int distScelto, benzinaRichiesta;  // il distributore scelto e la richiesta

		while (true) {
			// scelgo il distributore
			distScelto = (int) (Math.random() * dist.length);
			// scelgo la quantita' di benzina
			benzinaRichiesta = (int) (Math.random() * maxRich + 1);

			if ( dist[distScelto].carica(benzinaRichiesta) )
				System.out.println(getName() + ": carico   " +
                             benzinaRichiesta + " da " + dist[distScelto].nome);
			else
				System.out.println(getName() + ": rinuncia " +
                             benzinaRichiesta + " da " + dist[distScelto].nome);
         // uso una sleep per simulare il consumo di carburante
			try {
				Thread.sleep((int) (Math.random()*3000));
			} catch (InterruptedException e) {  }
		}
	}
}

class Cisterna extends Thread {
	Distributore[] dist;

	public Cisterna (Distributore[] d) {
		this.dist = d;
	}

	public void run () {
		while (true) {
			try {
				double i = Math.random();
				Thread.sleep ((int) (i*5000));
			} catch (InterruptedException e) { }

			// richiedo i distributori
			for(int i=0; i<dist.length; i++) {
				dist[i].acquisisci();
			}

			// li rifornisco
			for(int i=0; i<dist.length; i++) {
				dist[i].rifornimento();
			}

			// li rilascio
			for(int i=0; i<dist.length; i++) {
				dist[i].rilascia();
			}
		}
	}
}

class Distributore {
	public final String nome;
	final 	int pieno = 7;
	int capacita = pieno;
	boolean libero = true;

	public Distributore (String nome) {
		this.nome = nome;
	}

	public synchronized boolean carica (int richiesta) {
		while (!libero) // se c'e' rifornimento aspetto
			try {wait();} catch (InterruptedException e) { }

		if ( capacita < richiesta ) {
			return false;
		} else {
			capacita -= richiesta;
			return true;
		}
	}

	public synchronized void acquisisci () {
		// non ho bisogno di test: se entro vuol dire che non ci sono clienti
		// domanda: perche'?
		libero = false;
	}

	public synchronized void rifornimento () {
		System.out.println("Rifornimento " + nome);
		capacita = pieno;
	}

	public synchronized void rilascia () {
		// sveglio eventuali clienti in attesa
		libero = true;
		notifyAll();
		// domanda: si potrebbe usare solo notify()?
	}
}

